/*******************************************************************************
* Filename:     crc.c
*
* CRC7/CRC16̌vZ
*
*********************************************************************************/

#include "sdconfig.h"
#include    "drive.h"
#include "sdapi.h"

/* CRC calculation parameters */
#define         CRC_LEN         0x07
#define         CRC_POLYN       0x89
#define         CRC_HIGHBIT     0x80
#define         CRC_BITS        0x7F

#define         D_CRC_LEN       0x10
#define         D_CRC_POLYN     0x11021L
#define         D_CRC_HIGHBIT   0x10000L
#define         D_CRC_BITS      0x0FFFFL
#define         WRONG_DATA_CRC16        0x08
#define         WRONG_CMD_RESPONSE      0x04


#define CMD_BYTE_LENGTH         6       /* Length in bytes of MMC commands  */


#if (USE_SET_FEATURES )

/*******************************************************************************
* Name: calculateDataCRC16
*
*  CRC16vZ
*
********************************************************************************/
UINT16 calculateDataCRC16 ( UCHAR *pData, UINT16 data_length )
{
	UCHAR   *ppData;
	UINT32  rreg, rtemp;
	UINT16  ibyte, ibit;

	rreg = 0L;
	ppData = (UCHAR *)pData;

	for (ibyte = 0; ibyte < data_length; ibyte++)
	{
		rtemp = ((UINT32)ppData[ibyte]) << (D_CRC_LEN - 7);
		for (ibit = 0; ibit < 8; ibit++)
		{
			rreg <<= 1;
			rreg ^= ( ((rtemp ^ rreg) & D_CRC_HIGHBIT) ? D_CRC_POLYN : 0);
			rtemp <<= 1;
		}
	}

	return ((UINT16)(rreg & D_CRC_BITS));
}


/*******************************************************************************
* Name: calculateDataCRC7
*
*  CRC7vZ
*
********************************************************************************/
UINT16 calculateCRC7(UCHAR *dBuff, UINT16 offset, UINT16 dLength, SDBOOL cmdORresp)
{
	UINT16  ibyte, ibit;
	UINT16  rreg, crc_byte, dtmp;

	if ( cmdORresp )
		crc_byte = CRC_POLYN;
	else
		crc_byte = 0x09;

	rreg = 0;
	for (ibyte = offset; ibyte < (dLength - 1); ibyte++)
	{
		dtmp = (UINT16)(dBuff[ibyte]); 
		for (ibit = 0; ibit < 8; ibit++)
		{
			rreg <<= 1;
			dtmp &= 0xFF;
			rreg ^= (((dtmp ^ rreg) & CRC_HIGHBIT) ? crc_byte : 0);
			dtmp <<= 1;
		}
	}

	return (rreg&0xFF);
}

/*******************************************************************************
* Name: checkResponseCRC
*
*  MX|Xg[NCRC`FbN
*
********************************************************************************/
UINT16 checkResponseCRC ( UCHAR *response, UINT16 offset, UINT16 response_length )
{
	UINT16  crc_byte;

	crc_byte = calculateCRC7(response, offset, response_length, YES);
	crc_byte = (((crc_byte & CRC_BITS) << 1) | 0x01);

	if ( crc_byte != response[(response_length - 1)])
		return WRONG_CMD_RESPONSE;

	return (MMC_NO_ERROR);
}


/*******************************************************************************
* Name: appendCmdCRC
*
* R}hg[NɕtCRC7rbgvZ
*
********************************************************************************/
UINT16 appendCmdCRC ( UCHAR *cmd_bytes, UINT16 Length )
{
	UINT16   rreg;

	rreg = calculateCRC7(cmd_bytes, 0, Length, NO);
	return (((rreg << 1) | 0x01));

}

#endif /* #if (USE_SET_FEATURES ) */

/*******************************************************************************
* Name: calculateCmdCRC
*
* R}hCfbNXAA[MgCRCrbgvZ
*
********************************************************************************/
UINT16 calculateCmdCRC ( UINT32 args, UINT16 cmd_index )
{
	UINT16  i;
	UCHAR   cmd_bytes[CMD_BYTE_LENGTH];


	cmd_bytes[0] = (UCHAR)(cmd_index);

	/* MSB first */
	for (i = 4; i > 0; i--)
	{
		cmd_bytes[i] = (UCHAR)(args & 0xFFL);
		args >>= 8;
	}

#if (!USE_SET_FEATURES)
	/* CRCی얢gp̏ꍇACMD0g[N͏ɃrbgȂ̂Œ萔ŎĂ */
	if ( cmd_index == 0x40 ) /* startbit + CMD index */
		i = 0x95; /* CMD0̏ꍇ */
	else
		i = 0xFF; /* CMD0ȊO͖ */
#else
	i = appendCmdCRC(cmd_bytes, CMD_BYTE_LENGTH);@/* vZ */
#endif

	return i;
}


