/*	$NetBSD: vfs_bio.c,v 1.81 2002/05/12 23:06:27 matt Exp $	*/

/*-
 * Copyright (c) 1994 Christopher G. Demetriou
 * Copyright (c) 1982, 1986, 1989, 1993
 *	The Regents of the University of California.  All rights reserved.
 * (c) UNIX System Laboratories, Inc.
 * All or some portions of this file are derived from material licensed
 * to the University of California by American Telephone and Telegraph
 * Co. or Unix System Laboratories, Inc. and are reproduced herein with
 * the permission of UNIX System Laboratories, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)vfs_bio.c	8.6 (Berkeley) 1/11/94
 */
/*======================================================================*/
/*  LЁ@xm݌v          Fujisekk LTD                            */
/*  http://www.d1.dion.ne.jp/~fujisekk/             All Alound          */
/*  http://homepage3.nifty.com/fujisekk/            Library             */
/* TEL Japan 042-323-2217                                               */
/*  2005.6 Oomagari Modefy for Furtos                                   */
/*======================================================================*/
/*
 * Some references:
 *	Bach: The Design of the UNIX Operating System (Prentice Hall, 1986)
 *	Leffler, et al.: The Design and Implementation of the 4.3BSD
 *		UNIX Operating System (Addison Welley, 1989)
 */
#include "sdconfig.h"
#include "sdtypes.h"


#include <sys/cdefs.h>
#include <sys/param.h>

#include <sys/errno.h>

#include <sys/queue.h>
#include <sys/buf.h>
#include <sys/time.h>

#include <sys/vnode.h>
#include <sys/resource.h>
#include <sys/resourcevar.h>
#include <sys/proc.h>

#include "malloc.h"

#include <sys/ucred.h>

#include "mount.h"

#include    "drive.h"
#include "sdapi.h"
/*Main.c*/
extern unsigned int pbr_lba; /*Partition top*/

/* Macros to clear/set/test flags. */
#define	SET(t, f)	(t) |= (f)
#define	CLR(t, f)	(t) &= ~(f)
#define	ISSET(t, f)	((t) & (f))

/*
 * Definitions for the buffer hash lists.
 */
#define	BUFHASH(dvp, lbn)	\
	(&bufhashtbl[(((long)(dvp) >> 8) + (int)(lbn)) & bufhash])
LIST_HEAD(bufhashhdr, buf) *bufhashtbl, invalhash;
u_long	bufhash;
#ifndef SOFTDEP
//???struct bio_ops bioops;	/* I/O operation notification */
#endif

/*
 * Insq/Remq for the buffer hash lists.
 */
#define	binshash(bp, dp)	LIST_INSERT_HEAD(dp, bp, b_hash)
#define	bremhash(bp)		LIST_REMOVE(bp, b_hash)

/*
 * Definitions for the buffer free lists.
 */
#define	BQUEUES		4		/* number of free buffer queues */

#define	BQ_LOCKED	0		/* super-blocks &c */
#define	BQ_LRU		1		/* lru, useful buffers */
#define	BQ_AGE		2		/* rubbish */
#define	BQ_EMPTY	3		/* buffer headers with no memory */

TAILQ_HEAD(bqueues, buf) bufqueues[BQUEUES];
int needbuffer;

/*
 * Buffer pool for I/O buffers.
 */
//struct pool bufpool;

/*
 * Insq/Remq for the buffer free lists.
 */
#define	binsheadfree(bp, dp)	TAILQ_INSERT_HEAD(dp, bp, b_freelist)
#define	binstailfree(bp, dp)	TAILQ_INSERT_TAIL(dp, bp, b_freelist)

int count_lock_queue __P((void));

/* Local Buffer */
/* Block read With Bufffer Max 128Kb */ 
static struct buf1 buf_p;
struct buf1 buf_pw;




/*
 * Initialize buffers and hash links for buffers.
 */
/*void
bufinit()
{
	struct buf *bp;
	struct bqueues *dp;
	int i;
	int base, residual;
*/

/*
 * Read a disk block.
 * This algorithm described in Bach (p.54).
 */
/*int
bread(vp, blkno, size, cred, bpp)
	struct vnode *vp;
	daddr_t blkno;
	int size;
	struct ucred *cred;
	struct buf1 **bpp;
{
	struct bu1f *bp;

*/	/* Get buffer for block. */
/*	bp = *bpp = bio_doread(vp, blkno, size, cred, 0);

*/	/* Wait for the read to complete, and return result. */
/*	return (biowait(bp));
}
*/
/* read  partitions logilal  sectors */
/* buf1 return buffer addres */
int	bread(unsigned long bn ,unsigned long bsize,  struct buf1 **bp_i)
{
		int ret;
		unsigned short bcnt;
		struct buf1 *bp;
		bp=bp_i;
		bcnt=bsize/512;
			//error = bread(pmp->pm_devvp, bn, bsize, NOCRED, &bp);
	ret=spi_read(pbr_lba+bn, buf_p.b_dta, bcnt) ; /* ZN^̃[h */
//	if (!ISSET(bp->b_flags, (B_DONE | B_DELWRI))) {
		/* Start I/O for the buffer. */
		//SET(bp->b_flags, B_READ | async);
//		(bp->b_flags)|=( B_READ );
//	}
	//???	VOP_STRATEGY(bp);
	if(ret){
		(buf_p.b_flags)=( B_READ );
		*bp_i=&buf_p;
		return(0);
	}
	else{return(-1);}
}



/*
 * Block write.  Described in Bach (p.56)
}
*/
/* Write partitions logilal  sectors */
 int	bwrite_i(unsigned long bn ,unsigned long bsize,  char *bp)
{
		int ret;
		unsigned short bcnt;

		bcnt=bsize/512;
			//error = bread(pmp->pm_devvp, bn, bsize, NOCRED, &bp);
	ret=spi_write(pbr_lba+bn, buf_p.b_dta, bcnt) ;  /* ZN^ Write */
	if(ret){
		return(0);
	}
	else{return(-1);}
}
int	bwrite(struct buf1 *bp)
 
{
	int ans;
	ans=bwrite_i(bp->b_lblkno ,bp->b_bcount,  bp->b_dta);
	return(ans);
}
/*
 * Release a buffer on to the free lists.
 * Described in Bach (p. 46).
 */
void
brelse(bp)
	struct buf1 *bp;
{
	struct bqueues *bufq;
//	int s;

//???	KASSERT(ISSET(bp->b_flags, B_BUSY));

	/* Wake up any processes waiting for any buffer to become free. */
	if (needbuffer) {
		needbuffer = 0;
		//wakeup(&needbuffer);
	}

	/* Block disk interrupts. */
	//??? s = splbio();

	/* Wake up any proceeses waiting for _this_ buffer to become free. */
	if (ISSET(bp->b_flags, B_WANTED)) {
		CLR(bp->b_flags, B_WANTED|B_AGE);
//		wakeup(bp);
	}

	/*
	 * Determine which queue the buffer should be on, then put it there.
	 */

	/* If it's locked, don't report an error; try again later. */
	if (ISSET(bp->b_flags, (B_LOCKED|B_ERROR)) == (B_LOCKED|B_ERROR))
		CLR(bp->b_flags, B_ERROR);

	/* If it's not cacheable, or an error, mark it invalid. */
	if (ISSET(bp->b_flags, (B_NOCACHE|B_ERROR)))
		//SET(bp->b_flags, B_INVAL);
		(bp->b_flags)=( B_INVAL);

	if (ISSET(bp->b_flags, B_VFLUSH)) {
		/*
		 * This is a delayed write buffer that was just flushed to
		 * disk.  It is still on the LRU queue.  If it's become
		 * invalid, then we need to move it to a different queue;
		 * otherwise leave it in its current position.
		 */
		CLR(bp->b_flags, B_VFLUSH);
		if (!ISSET(bp->b_flags, B_ERROR|B_INVAL|B_LOCKED|B_AGE))
			goto already_queued;
		//???	else
		//???	bremfree(bp);
	}

	if ((bp->b_bufsize <= 0) || ISSET(bp->b_flags, B_INVAL)) {
		/*
		 * If it's invalid or empty, dissociate it from its vnode
		 * and put on the head of the appropriate queue.
		 */
//???		if (LIST_FIRST(&bp->b_dep) != NULL && bioops.io_deallocate)
	//???		(*bioops.io_deallocate)(bp);
		//???		CLR(bp->b_flags, B_DONE|B_DELWRI);
	//???			if (bp->b_vp) {
	//???				reassignbuf(bp, bp->b_vp);
	//???				brelvp(bp);
		//???		}
		if (bp->b_bufsize <= 0)
			/* no data */
			bufq = &bufqueues[BQ_EMPTY];
		else
			/* invalid data */
			bufq = &bufqueues[BQ_AGE];
		//??? binsheadfree(bp, bufq);
	} else {
		/*
		 * It has valid data.  Put it on the end of the appropriate
		 * queue, so that it'll stick around for as long as possible.
		 * If buf is AGE, but has dependencies, must put it on last
		 * bufqueue to be scanned, ie LRU. This protects against the
		 * livelock where BQ_AGE only has buffers with dependencies,
		 * and we thus never get to the dependent buffers in BQ_LRU.
		 */
		if (ISSET(bp->b_flags, B_LOCKED))
			/* locked in core */
			bufq = &bufqueues[BQ_LOCKED];
		else if (!ISSET(bp->b_flags, B_AGE))
			/* valid data */
			bufq = &bufqueues[BQ_LRU];
		else {
			/* stale but valid data */
			int has_deps;

	//???				if (LIST_FIRST(&bp->b_dep) != NULL &&
			    //???		bioops.io_countdeps)
			//???			has_deps = (*bioops.io_countdeps)(bp, 0);
	//???		else
				has_deps = 0;
			bufq = has_deps ? &bufqueues[BQ_LRU] :
			    &bufqueues[BQ_AGE];
		}
//???		binstailfree(bp, bufq);
	}

already_queued:
	/* Unlock the buffer. */
	CLR(bp->b_flags, B_AGE|B_ASYNC|B_BUSY|B_NOCACHE|B_ORDERED);
	//SET(bp->b_flags, B_CACHE);
	(bp->b_flags)|=( B_CACHE);

	/* Allow disk interrupts. */
	//???splx(s);
}

/*
 * Find a buffer which is available for use.
 * Select something from a free list.
 * Preference is to AGE list, then LRU list.    
 */
struct buf1 *
getnewbuf(slpflag, slptimeo)
	int slpflag, slptimeo;
{
	struct buf1 *bp;
	//??? int s;

start:
	//??? s = splbio();
//	if (ISSET(bp->b_flags, B_VFLUSH)) {
		/*
		 * This is a delayed write buffer being flushed to disk.  Make
		 * sure it gets aged out of the queue when it's finished, and
		 * leave it off the LRU queue.
		 */
//		CLR(bp->b_flags, B_VFLUSH);
		//SET(bp->b_flags, B_AGE);
//		(bp->b_flags)=( B_AGE);
		//???splx(s);
//		goto start;
//	}

	/* Buffer is no longer on free lists. */
	//SET(bp->b_flags, B_BUSY);
	(bp->b_flags)|=( B_BUSY);

	/*
	 * If buffer was a delayed write, start it and return NULL
	 * (since we might sleep while starting the write).
	Ƃ肠sp(ݑOɃLbVmۂ悤Ƃ̂)
	 */
	if (ISSET(bp->b_flags, B_DELWRI)) {
		//???splx(s);
		/*
		 * This buffer has gone through the LRU, so make sure it gets
		 * reused ASAP.
		 */
		//SET(bp->b_flags, B_AGE);
		(bp->b_flags)|=(B_AGE);
		bwrite_i(bp->b_lblkno, bp,1);
		return (NULL);
	}

	/* disassociate us from our vnode, if we had one... */
//???	if (bp->b_vp)
	//???	brelvp(bp);
	//???splx(s);

//???	if (LIST_FIRST(&bp->b_dep) != NULL && bioops.io_deallocate)
//???		(*bioops.io_deallocate)(bp);

	/* clear out various other fields */
	bp->b_flags = B_BUSY;
	bp->b_dev = NODEV;
	bp->b_blkno = bp->b_lblkno = bp->b_rawblkno = 0;
//	bp->b_iodone = 0;
	bp->b_error = 0;
	bp->b_resid = 0;
	bp->b_bcount = 0;
	
	//???bremhash(bp);
	return (bp); 
}


/*
 * Return a count of buffers on the "locked" queue.
 */
/*int
count_lock_queue()
{
	struct buf *bp;
	struct buf1 *bp;
	int n = 0;

	for (bp = bufqueues[BQ_LOCKED].tqh_first; bp;
	    bp = bp->b_freelist.tqe_next)
		n++;
	return (n);
}
*/
#ifdef DEBUG
/*
 * Print out statistics on the current allocation of the buffer pool.
 * Can be enabled to print out on every ``sync'' by setting "syncprt"
 * in vfs_syscalls.c using sysctl.
 */
void
vfs_bufstats()
{
	//??? int s, i, j, count;
	struct buf1 *bp;
	struct bqueues *dp;
	int counts[(MAXBSIZE / PAGE_SIZE) + 1];
	static char *bname[BQUEUES] = { "LOCKED", "LRU", "AGE", "EMPTY" };

	for (dp = bufqueues, i = 0; dp < &bufqueues[BQUEUES]; dp++, i++) {
		count = 0;
		for (j = 0; j <= MAXBSIZE/PAGE_SIZE; j++)
			counts[j] = 0;
		//??? s = splbio();
		for (bp = dp->tqh_first; bp; bp = bp->b_freelist.tqe_next) {
			counts[bp->b_bufsize/PAGE_SIZE]++;
			count++;
		}
		//???splx(s);
		printf("%s: total-%d", bname[i], count);
		for (j = 0; j <= MAXBSIZE/PAGE_SIZE; j++)
			if (counts[j] != 0)
				printf(", %d-%d", j * PAGE_SIZE, counts[j]);
		printf("\n");
	}
}
#endif /* DEBUG */
