/********************************************************************************
* Filename:    sdmmc.c
*
* MMCSPIR}hV[PX
*
********************************************************************************/

#include "sdconfig.h"
#include    "drive.h"
#include "sdmmc.h"

/*******************************************************************************
* Name: mmcReset
*
*  MMCJ[hSPI[hֈڍsAMMCJ[hIy[V\Ԃɂ
* 
********************************************************************************/
MMC_CC mmcReset( PDEVICE_CONTROLLER pc )
{
	MMC_CC  resErr;

	pc->timer = RESET_DELAY << 2;

	/* CS->Low CMD0sSPI[hֈڍs */
	selectChip();
	resErr = mmcCommandAndResponse ( pc,
				GO_IDLE_STATE,
				0L,
				0,
				R1 );

	deSelectChip();

	/* ̎_ł́AIn Idle StateX|XԂĂ͂ */
	if ( (resErr != MMC_CARD_IS_NOT_READY) || !resErr )
	{
		pc->error_code = MMC_INTERFACE_ERROR;
		return MMC_INTERFACE_ERROR;
	}

	/* CMD1J[hIdleXe[g𔲂܂Ŕs */
	while ( pc->timer )
	{
		selectChip();
		resErr = mmcCommandAndResponse ( pc,
					SEND_OP_COND,
					0L,
					0,
					R1 );
		deSelectChip ();

		/* In Idle StateȊÕX|XʒmꂽꍇG[ */
		if ( (resErr != MMC_CARD_IS_NOT_READY) && resErr )
		{
			pc->error_code = (UINT16)resErr;
			return resErr;
		}

		if ( resErr == MMC_CARD_IS_NOT_READY )
		{
			pc->timer--;
			OS_WAIT(1);     	/* CMD1s܂ 1ms@̃fBC */
			if ( !pc->timer ) 	/* gC񐔃I[o[ */
			{
				pc->error_code = MMC_CARD_IS_NOT_READY;
				return MMC_CARD_IS_NOT_READY;
			}
		}
		else /* Idle State𔲂 */
			break;
	}

	pc->error_code = MMC_NO_ERROR;
	return MMC_NO_ERROR;
}



/*******************************************************************************
* Name: mmcGetCardIdentification
*
*  CIDWX^l̊l
*
********************************************************************************/
MMC_CC mmcGetCardIdentification ( PDEVICE_CONTROLLER pc, UCHAR *mmcIdent )
{

	MMC_CC  resErr;

	pc->user_address = (USERADDRESS)mmcIdent;

	selectChip();

	resErr = mmcCommandAndResponse ( /* CMD10 (SEND_CID)R}h𔭍s */
				pc,
				SEND_CID, 
				0L,
				0,
				R1 );
	if ( resErr == MMC_NO_ERROR )
		resErr = receive_data( pc, (CID_BYTE_LENGTH - 1) ); /* CIDWX^l̎M */

	deSelectChip ();

	pc->error_code = (UINT16)resErr;
	return resErr;
}



/*******************************************************************************
* Name: mmcGetConfiguration
* 
*  CSDWX^l̊l
* 
********************************************************************************/
MMC_CC mmcGetConfiguration( PDEVICE_CONTROLLER pc,
			UCHAR *respCSD )
{
	MMC_CC  resErr;


	pc->user_address = (USERADDRESS)respCSD;

	selectChip();

	resErr = mmcCommandAndResponse ( /* CMD9 (SEND_CSD)R}h𔭍s */
				pc,
				SEND_CSD, 
				0L,
				0,
				R1 );
	if ( resErr == MMC_NO_ERROR )
		resErr = receive_data( pc, (CID_BYTE_LENGTH - 1) );

	deSelectChip ();


	pc->error_code = (UINT16)resErr;
	return resErr;
}


/*******************************************************************************
* Name: mmcBlkLengthConfiguration
*
*  f[^ubN̐ݒ
*
************************************************************************************/
MMC_CC mmcBlkLengthConfiguration( PDEVICE_CONTROLLER pc )
{
	MMC_CC  resErr;

	selectChip();

	resErr = mmcCommandAndResponse ( /* CMD16(SET_BLOCKLEN)R}h𔭍sāAubNݒ */
				pc,
				SET_BLOCKLEN, 
				(ULONG)pc->block_size,
				0,
				R1 );

	deSelectChip();

	pc->error_code = (UINT16)resErr;
	return (resErr);
}
  

/***********************************************************************************
* Name: mmcRead - Reads one sector from card
*                                                                                                                                                                       |
*  SPI[hPubNf[^[h
*
************************************************************************************/
MMC_CC mmcRead( PDEVICE_CONTROLLER pc, ULONG dataAddress )
{
	MMC_CC  resErr;

	selectChip();

	resErr = mmcCommandAndResponse ( /* CMD17 (READ_SINGLE_BLOCK) 𔭍săX|X󂯎 */
				pc,
				READ_BLOCK, 
				(dataAddress * (ULONG)pc->block_size), /* [hJnAhXubNEɂ */
				1,
				R1 );
	if (resErr == MMC_NO_ERROR)
	{
		resErr = receive_data( pc, pc->block_size ); /* f[^ubN̎M */
	}

	deSelectChip();

	pc->error_code = (UINT16)resErr;
	return resErr;
}



/***********************************************************************************
* Name: mmcWrite
*
* SPI[hPubNf[^Cg
*
************************************************************************************/
MMC_CC mmcWrite ( PDEVICE_CONTROLLER pc, ULONG dataAddress )
{
	MMC_CC  resErr;


	selectChip();

	resErr = mmcCommandAndResponse (	/* CMD24 (WRITE_BLOCK)𔭍s */
				pc,
				WRITE_BLOCK, 
				(dataAddress * (ULONG)pc->block_size), /* CgJnAhXubNEɂ */
				1,
				R1 );
	if ( resErr == MMC_NO_ERROR )
		resErr = send_data( pc, pc->block_size ); /* WRITEf[^ubNJ[h֑M */

	deSelectChip();

	pc->error_code = (UINT16)resErr;
	return resErr;
}



/***********************************************************************************
* Name: mmcGetStatus
*
* MMCJ[hXe[^Xl̊l
*
************************************************************************************/
MMC_CC mmcGetStatus ( PDEVICE_CONTROLLER pc )
{
	MMC_CC  resErr;


	selectChip ();

	resErr = mmcCommandAndResponse ( pc,
				SEND_STATUS, /* CMD13 */
				0L,
				0,
				R2 );

	deSelectChip();

	pc->error_code = (UINT16)resErr;
	return (resErr);
}


#if (USE_SET_FEATURES)
/***********************************************************************************
* Name: mmcTurnCRCOnOff - (SPI mode only)
*
*  f[^CCRCیON/OFF
*
************************************************************************************/
MMC_CC mmcTurnCRCOnOff( PDEVICE_CONTROLLER pc, UINT16 opt )
{
	MMC_CC  resErr;

	selectChip ();

	resErr = mmcCommandAndResponse ( pc,
				CRC_ON_OFF, 
				(ULONG)opt,
				0,
				R1 );

	deSelectChip ();

	pc->error_code = (UINT16)resErr;
	return (resErr);
}

#endif  /* (USE_SET_FEATURES) */


