/********************************************************************************
* Filename: sdmmc.h
*
*  }`fBAJ[h R}h֐vg^Cv
*
********************************************************************************/

#ifndef _SDMMC_H_

#include "sdapi.h"
#include "drive.h"

#define DEFAULT_BLK_LEN         512		/* Data block length */
#define BYTE_LENGTH             6       /* Length in bytes of MMC responses */
#define CID_BYTE_LENGTH         17      /* Length in bytes of MMC card id. CID */
#define CMD_BYTE_LENGTH         6       /* Length in bytes of MMC commands  */
#define RESET_DELAY             30      /* Delay for reset */

/* ------------------------- CMD's names: ---------------------------- */
/* ------------------ MMCR}hCfbNX -------------------------*/
#define CMD_START_BIT           0x40 /* Start bit */
#define GO_IDLE_STATE           (0 + CMD_START_BIT)
#define SEND_OP_COND            (1 + CMD_START_BIT)
#define ALL_SEND_CID            (2 + CMD_START_BIT)
#define SET_RELATIVE_ADDR       (3 + CMD_START_BIT)
#define SET_DSR                 (4 + CMD_START_BIT)
#define SELECT_DESELECT_CARD    (7 + CMD_START_BIT)
#define SEND_CSD                (9 + CMD_START_BIT)
#define SEND_CID                (10 + CMD_START_BIT)
#define READ_DAT_UNTIL_STOP     (11 + CMD_START_BIT)
#define STOP_TRANSMISSION       (12 + CMD_START_BIT)
#define SEND_STATUS             (13 + CMD_START_BIT)
#define SET_BUS_WIDTH_REGISTER  (14 + CMD_START_BIT)
#define GO_INACTIVE_STATE       (15 + CMD_START_BIT)
#define SET_BLOCKLEN            (16 + CMD_START_BIT)
#define READ_BLOCK              (17 + CMD_START_BIT)
#define READ_MULTIPLE_BLOCK     (18 + CMD_START_BIT)
#define WRITE_DAT_UNTIL_STOP    (20 + CMD_START_BIT)
#define WRITE_BLOCK             (24 + CMD_START_BIT)
#define WRITE_MULTIPLE_BLOCK    (25 + CMD_START_BIT)
#define PROGRAM_CID             (26 + CMD_START_BIT)
#define PROGRAM_CSD             (27 + CMD_START_BIT)
#define SET_WRITE_PROT          (28 + CMD_START_BIT)
#define CLR_WRITE_PROT          (29 + CMD_START_BIT)
#define SEND_WRITE_PROT         (30 + CMD_START_BIT)
#define TAG_SECTOR_START        (32 + CMD_START_BIT)
#define TAG_SECTOR_END          (33 + CMD_START_BIT)
#define UNTAG_SECTOR            (34 + CMD_START_BIT)
#define TAG_ERASE_GROUP_START   (35 + CMD_START_BIT)
#define TAG_ERASE_GROUP_END     (36 + CMD_START_BIT)
#define UNTAG_ERASE_GROUP       (37 + CMD_START_BIT)
#define ERASE_SECTORS           (38 + CMD_START_BIT)
#define CRC_ON_OFF              (59 + CMD_START_BIT)

/* MMC R}hX|X` */
typedef enum
{
	R0 = 0,         /* NONE response */
	R1 = 1,         /* R1 response. Basic MMC response format */
	R2,             /* R2 response. Used by SEND_STATUS(CMD13) in SPI mode */
	R3              /* R3 response. not used in SPI mode */
} RESP_TYPE;


/* MMC StatusWX^̃rbg}XN萔 */
#define OVERRUN                 0x8000
#define CMD_PARAM_ERROR         0x4000
#define ADDRESS_ERROR           0x2000
#define ERASE_SEQ_ERROR         0x1000
#define CMD_CRC_ERROR           0x0800
#define COMUNC_ILLEG_COM        0x0400
#define ERASE_RESET             0x0200
#define CARD_IS_NOT_READY       0x0100
#define CMD_PARAM_ERROR2        0x0080
#define ERASE_PARAM             0x0040
#define WP_VIOLATION            0x0020
#define CARD_READ_FAILURE       0x0010
#define EERROR                  0x0008
#define ERROR2                  0x0004
#define WP_ERASE_SKIP           0x0002
#define UNDERRUN                0x0001


/*------- Token ʐM֐ ---------------------------*/
MMC_CC mmcCommandAndResponse(PDEVICE_CONTROLLER pc, UINT16 Cmd, UINT32 Arg, UINT16 noBlocks, RESP_TYPE Resp );
MMC_CC receive_data(PDEVICE_CONTROLLER pc, UINT16 data_length);
MMC_CC send_data(PDEVICE_CONTROLLER pc, UINT16 data_length);
#define selectChip()  (spi_cs_enable())
#define deSelectChip() (spi_cs_disable())
MMC_CC IsCardBusy(PDEVICE_CONTROLLER pc);


/*------- Command V[PX֐ ---------------------------*/
MMC_CC mmcReset ( PDEVICE_CONTROLLER pc );
MMC_CC mmcGetConfiguration ( PDEVICE_CONTROLLER pc, UCHAR *mmcCSD );
MMC_CC mmcGetCardIdentification( PDEVICE_CONTROLLER pc, UCHAR *mmcIdent );
MMC_CC mmcBlkLengthConfiguration( PDEVICE_CONTROLLER pc );
MMC_CC mmcRead ( PDEVICE_CONTROLLER pc, ULONG dataAddress );
MMC_CC mmcWrite ( PDEVICE_CONTROLLER pc, ULONG dataAddress );
MMC_CC mmcGetStatus( PDEVICE_CONTROLLER pc );
//extern unsigned int pbr_lba; /*Partition top*/
#if (USE_SET_FEATURES)
MMC_CC mmcTurnCRCOnOff( PDEVICE_CONTROLLER pc, UINT16 opt );
#endif /* (USE_SET_FEATURES) */


#define _SDMMC_H_
#endif


