/*****************************************************************************
* FileName: SPI.C
*
*   SPI[h API`
*
****************************************************************************/
#include "sdconfig.h"
#include "sdtypes.h"
#include    "drive.h"
#include "disklabel.h"
#include "sdmmc.h"


/*****************************************************************************/
/*****************************************************************************/
/******* SPI software layer.  ʃ\tgwւAPI ****************************/
/*****************************************************************************/
/*****************************************************************************/


/*****************************************************************************
* Name: spi_init
*
* MMCzXgAhCoAMMCJ[h̏
*
*****************************************************************************/
SDBOOL spi_init( SDVOID ) /*__fn__*/
{
	/* hCo\̂ */
	clear_controller_structure();

	/* SPIzXgH/W */
	spi_hw_init();

	/* MMCNbN̑o */
	startMMC80Clocks();

	return (YES);
}


/*****************************************************************************
* Name: spi_drive_open
*
* MMCJ[h̃I[v
*
*****************************************************************************/
DEVICE_CONTROLLER *  spi_drive_open(SDVOID) /*__fn__*/
//SDBOOL  spi_drive_open(SDVOID) /*__fn__*/
{
	PDEVICE_CONTROLLER pc;
	ULONG   ltemp;
	UINT16  heads;
	UINT16  sectors;
	UCHAR   buf[128];

	/* J[hRg[\ */
	pc = &controller_s;

	if (!pc)
		return(0);

	if ( !pc->opencount )
	{
		pc->controller_number = 0;
		pc->opencount = YES;
	}


	/* MMCSPI[hŃZbg */
	if( mmcReset(pc) != MMC_NO_ERROR )
		return(0);

#if (USE_SET_FEATURES)
	/* CRCgpꍇACMD59𔭍sONɂ */
	if ( mmcTurnCRCOnOff(pc, YES) )
		return(0);
#endif

	/* MMCCIDWX^lCMD10Ŋl */
	if ( mmcGetCardIdentification(pc, buf) != MMC_NO_ERROR )
		return(0);

	/* MMCCSDWX^lCMD9Ŋl */
	if ( mmcGetConfiguration(pc, (buf+64)) != MMC_NO_ERROR )
		return(0);

	/* CSDWX^lf[^ubNl */
	heads = (((UINT16)buf[64+5]) & 0x0F); /* READ_BL_LEN */
	sectors = (UINT16)(1 << heads); /* f[^ubN(Byte) */

	/* Read/Write f[^ubNftHg512Byteɂ */
	/* ubNقȂꍇA512Byteɐݒ */
	if ( sectors != DEFAULT_BLK_LEN )
	{
		pc->block_size = DEFAULT_BLK_LEN;

		/* CMD16(SET_BLOCKLEN)R}h𔭍sāAubNݒ */
		if( mmcBlkLengthConfiguration( pc ) != MMC_NO_ERROR ) 
			return 0;

		/* ubNύX̂ŁACSDWX^lXV */
		if ( mmcGetConfiguration(pc, (buf+64)) != MMC_NO_ERROR )
			return(0);
	}
	pc->block_size = DEFAULT_BLK_LEN;

	/* CSDWX^lAJ[heʂvZ */
	sectors = (((UINT16)buf[64+6]) & 3);	/* C_SIZEtB[h̊l */
	sectors <<= 8;
	ltemp = ((ULONG)buf[64+7]) + (ULONG)sectors;
	ltemp <<= 2;
	sectors = ((UINT16)(buf[64+8]) >> 6) & 3;
	ltemp += ((ULONG)sectors + 1L);

	heads = ((UINT16)buf[64+9] & 3);	/* C_SIZE_MULT tB[h̊l */
	heads <<= 1;
	sectors = ((((UINT16)buf[64+10]) >> 7) & 1);

	/* g[^ubN BLOCKNR */
	ltemp <<= ((heads + sectors + 2)); /* (C_SIZE+1) <<= (C_SIZE_MULT+2) */

	/* ATAhCuG~[VpɃhCuWIg */
 	/* Logical Block Address(LBA) */
	pc->drive.total_lba = ltemp;
	pc->drive.supports_lba = YES;

	/* CHS */
	heads   = 2;            /* Number of heads */
	if ( ltemp > 0xFFFF )  /* 32 MB in size */
		heads = 4;
	sectors  = 32;          /* Number of sectors per track */
	ltemp = ltemp / (ULONG)(heads * sectors);
	pc->drive.max_multiple = 1;
	pc->drive.num_cylinders = (UINT16)ltemp;
	pc->drive.num_heads =  (UCHAR)heads;
	pc->drive.sec_p_track = (UCHAR)sectors;

	/* 1ZN^ 512Byte */
	pc->drive.block_size = pc->block_size;
	return (pc);
}
/*****************************************************************************
* Name: spi_drive_close
*
* MMCJ[hclose 
* now void
*****************************************************************************/

SDBOOL spi_drive_close (SDVOID) /*__fn__*/
{
	return (YES);
}


MMC_CC mmedia_io(ULONG sector, UTINY *buffer, UCOUNT scount, INT16 op );
/*****************************************************************************
* Name: spi_read
*
* numSectors ubN[h
*
*****************************************************************************/
SDBOOL spi_read(ULONG startingSector, UTINY *buff, UCOUNT numSectors) /*__fn__*/
{
	if (mmedia_io(startingSector, buff, numSectors, READING))
		return (NO);

	return(YES);
}


/*****************************************************************************
* Name: spi_write
*
* PubNCg
*
*****************************************************************************/
SDBOOL spi_write(ULONG startingSector, UTINY *buff, UCOUNT numSectors) /*__fn__*/
{
	if (mmedia_io(startingSector, buff, numSectors, WRITING))
		return (NO);

	return(YES);
}


/*********************************************************************************
* Name: mmedia_io
*
*  MMCւ̃f[^ Read/Write
*
**********************************************************************************/
MMC_CC mmedia_io(ULONG sector, UTINY *buffer, UCOUNT scount, INT16 op ) /*__fn__*/
{
	PDEVICE_CONTROLLER pc;
	ULONG   starting_sector;
	MMC_CC  ret_val;


	starting_sector = sector;
	pc = &controller_s;

	/* `FbN */
	if( !scount || !pc )
	{
		pc->error_code = MMC_ADDRESS_ERROR;
		return (MMC_ADDRESS_ERROR);
	}
	pc->block_size = pc->drive.block_size;

	/* ANZXGA̔ */
	if ( (starting_sector+scount) > pc->drive.total_lba )
	{
		pc->error_code = MMC_ADDRESS_ERROR;
		return (MMC_ADDRESS_ERROR);
	}

	ret_val = MMC_NO_ERROR;
	pc->sectors_remaining = scount;
	pc->user_address = (USERADDRESS)buffer;
	if (op == READING)
	{
		/* wZN^[h */
		while (pc->sectors_remaining)
		{
			ret_val = mmcRead(pc, starting_sector);

			if ( ret_val != MMC_NO_ERROR )
			{
				pc->error_code = (UINT16)ret_val;
				return ret_val;
			}

			pc->sectors_remaining--;
			starting_sector++;

			/* Next Block address */
			pc->user_address += pc->block_size;
		}
	}

	else if (op == WRITING)
	{
		/* wZN^Cg */
		while ( pc->sectors_remaining )
		{
			ret_val = mmcWrite( pc, starting_sector );

			if ( ret_val != MMC_NO_ERROR )
			{
				pc->error_code = (UINT16)ret_val;
				return (ret_val);
			}

			/* Next block */
			pc->sectors_remaining--;
			starting_sector++;
			pc->user_address += pc->block_size;
		}
	}

	pc->error_code = (UINT16)ret_val;
	return (ret_val);
}


