/*****************************************************************************
* FileName: spioem.c
*
*   SPIʐM@(MN1A7xxx SPIC^[tF[XhCoj
******************************************************************************/

#include    "drive.h"
#include "sdapi.h"
#include "sample.h"

/***************** MN1A7xxx Rg[WX^̒` ************************/
/***** Serial Pin Group2 *****/
#define	K2MD	((unsigned char  *)0x2000F000)  /* Mode */
#define K2PLU	((unsigned char  *)0x2000F004)  /* Pull up */
/***** Serial I/F 4 *****/
#define S4SSCR0 ((unsigned short *)0x20007000)  /* SSI4-Control1 */
#define S4SSCR1 ((unsigned short *)0x20007004)  /* SSI4-Control2 */
#define S4SSDR 	((unsigned short *)0x2000700C)  /* SSI4-Data */
#define S4SSSR  ((unsigned short *)0x20007014)  /* SSI4-Interrupt */

/* Control Register Bit Mask */
#define MD_SELECT_SSI4    0x0001    /* KnMD Select Exclusive Pin to SSI4 */
#define PLU_CTS_SCS       0x0001    /* KnPLU Pull-up nCTSn/nSCSn pin */
#define PLU_SIN           0x0002    /* KnPLU Pull-up SINn pin */
#define PLU_CLKIN         0x0004    /* KnPLU Pull-up SCLKINn pin */
#define CR0_SPI8BIT_SET   0x0007    /* SnSSCR0 Motorola SPI & 8bits */
#define CR0_SPI16BIT_SET  0x000F    /* SnSSCR0 Motorola SPI & 16bits */
#define CR0_EN_SSI        0x0080    /* SnSSCR0 Enable SSI */
#define CR0_DS_SSI        0xFF7F    /* SnSSCR0 Disable SSI (MASK) */
#define CR0_SCLKDIV64     0x3F00	/* SCLK (Src Clock / 64) */
#define CR0_SCLKDIV32     0x1F00	/* SCLK (Src Clock / 32) */
#define CR0_SCLKDIV16     0x0F00	/* SCLK (Src Clock / 16) */
#define CR0_SCLKDIV08     0x0700	/* SCLK (Src Clock / 8)  */
#define CR0_SCLKDIV04     0x0300	/* SCLK (Src Clock / 4)  */
#define CR0_SCLKDIV02     0x0100	/* SCLK (Src Clock / 2)  */
#define CR0_SCLKDIV01     0x0000	/* SCLK (Src Clock)      */
#define CR1_SETUP_BIT     0x0000    /* SnSSCR1 Setup Default */
#define SR_STS_TRNFIFO    0x0002    /* SnSSSR Status bit for Transfer FIFO */
#define SR_STS_RCVFIFO    0x0004    /* SnSSSR Status bit for Receive FIFO */
#define SR_STS_SSIBSY     0x0008    /* SnSSSR Status bit for SSI */
#define SR_STS_TRNREQ     0x0010    /* SnSSSR Transfer Request */
#define SR_STS_RCVREQ     0x0020    /* SnSSSR Receive Request */
#define SR_STS_OVERRUN    0x0040    /* SnSSSR Receive Overrun Status */
#define CS_DRV0_MASK      0x0001    /* Port3 CS emulate bit */

#define DATA_TOKEN            0xFE   /* f[^g[NX^[goCg */
#define DUMMY_DATA            0xFF   /* _~[f[^ */

SDLOCAL SDBOOL SPIExchangeData(UCHAR odata, UCHAR *iData);
UINT16 	SclkDiv;

/***********************************************************************************
* Name: spi_hw_init
* SPI[hł̒ʐMŎgp|[g̏
************************************************************************************/
SDVOID spi_hw_init(void) /*__fn__*/
{

	/* Port3 Rg[WX^ݒ */
	*P3MD  |= CS_DRV0_MASK; /* P3-Bit0ėp|[g֐ݒ */
	*P3DIR |= CS_DRV0_MASK;	/* P3-Bit0o̓|[gƂĐݒ */
	*P3PLU |= CS_DRV0_MASK; /* P3-Bit0vAbv */
	*P3OUT |= CS_DRV0_MASK; /* Hixo */
/*
*P3MD  |= 0x04;
*P3DIR |= 0x04;
*P3PLU |= 0x04;
*P3OUT |= 0x04;
*/
	/* SSI2SSI4̋ʃsASSI4gpɐ؂ւ */
	*K2MD = MD_SELECT_SSI4;

	/* VANbN̏̕ */
	SclkDiv = CR0_SCLKDIV64;

/* SSI4 Rg[WX^ɒʐMݒ */
*S4SSCR0 = (SclkDiv | CR0_SPI8BIT_SET); /* SPI Frame & 8bits transfer */
*S4SSCR1 = CR1_SETUP_BIT;				/* SPI clock timing & Select */
}


/***********************************************************************************
* Name: spi_cs_enable
* CSMLowxɂāAMMCIԂɂ
************************************************************************************/
SDVOID spi_cs_enable(void) /*__fn__*/
{
	/* }`fBAJ[hCSI(Lowx)ɂ */
	*P3OUT &= ~(CS_DRV0_MASK); /* Port3 Bit0̏o͒lOɂ */
/*
*P3OUT &= 0xFB;
*/
}


/***********************************************************************************
* Name: spi_cs_disable
* CSMHixɂāAMMCIԂɂ
************************************************************************************/
SDVOID spi_cs_disable( void ) /*__fn__*/
{
	UCHAR idata;

	/* R}ĥ߂WNbN */
	SPIExchangeData(DUMMY_DATA, &idata);

	/* }`fBAJ[hCS𖢑I(Hix)ɂ */
	*P3OUT |= CS_DRV0_MASK; /* Port3 Bit0o͒lPɂ */

}


/***********************************************************************************
* Name: SPIExchangeData
* SPIoXłPoCgf[^ʐMsB
************************************************************************************/
SDLOCAL SDBOOL SPIExchangeData(UCHAR odata, UCHAR *iData)
{
	UINT16  i;
	UCHAR   sts;
	UINT16 	fifo_data;
#if 0
	/* SSI4 Rg[WX^ɒʐMݒ */
	*S4SSCR0 = (SclkDiv | CR0_SPI8BIT_SET); /* SPI Frame & 8bits transfer */
	*S4SSCR1 = CR1_SETUP_BIT;				/* SPI clock timing & Select */
#endif

/*
*P3OUT |= 0x04;
*/
	/* SSI4 𓮍\ɂ@*/
	*S4SSCR0 |= CR0_EN_SSI;

	/* S4SSDR WX^֏o͒l݁AʐMJn */
	fifo_data = odata;
	fifo_data <<= 8;
	*S4SSDR = fifo_data;

	/* ]Jn|[OŔ (MN1A7xxxFIFOX^bN҂̂) */
	sts = *S4SSSR;
	i = 0;
	do{
		if(sts & SR_STS_SSIBSY) break;
		sts = *S4SSSR;
		i++;
	} while(i < 0xFF);

	/* ]|[OŔ */
	for(i=0; i < 0x40; i++) {
		if(!(*S4SSSR & SR_STS_SSIBSY)) break; /* S4SSSRWX^Busyrbg */
	}

	if(i == 0x40){	/* ^CAEg */
		*S4SSCR0 &= CR0_DS_SSI; /* SSI4 ~ */
		return (NO);
	}

	/* J[h̓̓f[^f[^WX^ǂݏo */
	fifo_data = *S4SSDR;
	*iData = (UCHAR)(fifo_data & 0xFF);

	*S4SSCR0 &= CR0_DS_SSI; /* SSI4 ~ */
	return (YES);
}


/**************************************************************************
* Name: startMMC80Clocks
* MMĈ߁A_~[80NbNo͂
**************************************************************************/
SDVOID startMMC80Clocks(void)
{
	UINT16  i;
	UCHAR   jj;

	/* J[h80NbNo */
	for (i = 0; i < 10; i++) /* 8bit * 10 = 80 */
		SPIExchangeData(DUMMY_DATA, &jj);
}


/***************************************************************************
* Name: SendMMCCommand
* R}hg[NMMC֑
***************************************************************************/
SDVOID mmcSendCommand(UINT32 Args, UINT16 command, UINT16 crcCMD)
{
	UCHAR   *cmd_ptr;
	UINT16  ii;
	UCHAR   jj;
	UCHAR   cmd_bytes[6];
/*
*P3OUT &= 0xFB;
*/

	/* R}hg[N̐ */
	cmd_bytes[0] = (UCHAR)(command); /* R}hԍ */
	cmd_bytes[5] = (UCHAR)crcCMD;    /* CRC7rbg */
	for (ii = 4; ii > 0; ii--){      /* R}h */
		cmd_bytes[ii] = (UCHAR)(Args & 0xFFL);
		Args >>= 8;
	}

	/* R}hg[N̑M */
	cmd_ptr = (UCHAR *)cmd_bytes;
	for (ii = 0; ii < 6; ii++) {
		if ( !SPIExchangeData(cmd_ptr[ii], &jj) )
			return;
	}
}


/***************************************************************************
* Name: getMMCResponseInfo
* MMCR}hX|Xg[NM
***************************************************************************/
MMC_CC getMMCResponseInfo(UINT16 *respStatus, UINT16 respType)
{
	UINT16  i;
	UCHAR   iData;

	for (i = 0; i < 0x400; i++) {
		iData = 0xFF;
		if ( !SPIExchangeData(DUMMY_DATA, &iData) )
			return MMC_INTERFACE_ERROR;

		iData &= 0xFF;
		if ( !(iData & 0x80) ) {   /* X|Xg[NM */
			*respStatus = ((UINT16)iData << 8);
			if ( respType == 2 ) { /* 2Byte^CṽX|Xg[N */
				if ( !SPIExchangeData(DUMMY_DATA, &iData) )
					return MMC_INTERFACE_ERROR;
				iData &= 0xFF;
				*respStatus |= ((UINT16)iData);
			}
			return MMC_NO_ERROR;
		}
	}

	return MMC_CARD_IS_NOT_RESPONDING;
}


/*****************************************************************************
* Name: isCRCErrOnWrite
* MMCf[^X|Xg[NM
*****************************************************************************/
MMC_CC isCRCErrOnWrite(SDVOID)
{
	UINT16  ii;
	UCHAR   dDat;

	ii = 0xFFF;
	while ( ii ) {
		if ( !SPIExchangeData(DUMMY_DATA, &dDat) )
			return YES;
		dDat &= 0x0F;
		if ( dDat == 0x05 )
			return MMC_NO_ERROR;// /* MMC̃f[^MɊ */
		if ( dDat == 0x0B )
			return MMC_DATA_STATUS_CRC_ERROR; /* CRCerror */
		ii--;
	}

	/* J[hf[^X|Xg[No͂ĂȂ */
	return MMC_DATA_STATUS_CRC_ERROR;
}


/***************************************************************************
* Name: MMCTransmit
* f[^g[NJ[h֑
***************************************************************************/
MMC_CC MMCTransmit(UCHAR *buff, UINT16 dLength)
{
	UCHAR   *buff_ptr;
	UINT16  ii;
	UCHAR   jj;

	/* f[^g[NEX^[goCg̑M */
	SPIExchangeData(DUMMY_DATA, &jj);
	SPIExchangeData(DATA_TOKEN, &jj);

	/* f[^M */
	buff_ptr = buff;
	for (ii = 0; ii < dLength; ii++) {
		if( !SPIExchangeData(*buff_ptr++, &jj) )
			return MMC_INTERFACE_ERROR;
	}

#if (USE_SET_FEATURES)
	/* CRC`FbNONɂĂꍇ́A16bit CRCvZ */
	ii = calculateDataCRC16( buff, dLength );
#endif

	/* f[^g[NE16bit CRC𑗐M */
	SPIExchangeData((UCHAR)(ii >> 8), &jj);
	SPIExchangeData((UCHAR)(ii & 0xFF), &jj);

	/* J[hf[^X|Xg[NM */
	if ( isCRCErrOnWrite() )
		return MMC_DATA_STATUS_CRC_ERROR;

	return MMC_NO_ERROR;
}


/***************************************************************************
* Name: MMCReceive
* f[^g[NMMCM
***************************************************************************/
MMC_CC MMCReceive(UCHAR *buff, UINT16 dLength)
{
	UCHAR  *buff_ptr;
	UINT16  ii;
	UCHAR   dToken;

    
	/* f[^g[NM҂ */
	ii = 0xFFFF;
	while ( ii ) {
		/* f[^g[NEX^[goCgM𔻒 */
		if ( !SPIExchangeData(DUMMY_DATA, &dToken) )
			return MMC_INTERFACE_ERROR;
		dToken &= 0xFF;
		if ( dToken == DATA_TOKEN )
			break;
		ii--;
	}

	if ( ii == 0 )
		return MMC_INTERFACE_ERROR;

	/* f[^M */
	buff_ptr = buff;
	for (ii = 0; ii < dLength; ii++) {
		if ( !SPIExchangeData(DUMMY_DATA, &dToken) )
			return MMC_INTERFACE_ERROR;
		*buff_ptr++ = dToken;
	}

	/* 16Bit CRC̎M */
	SPIExchangeData(DUMMY_DATA, &dToken);
	ii = ((UINT16)dToken) << 8;
	SPIExchangeData(DUMMY_DATA, &dToken);
	ii |= ((UINT16)dToken);

#if (USE_SET_FEATURES)
	/* CRC`FbNONɂĂꍇ́A`FbNs */
	if(ii != calculateDataCRC16( buff, dLength ) ) return MMC_DATA_CRC_ERROR;
#endif

	return MMC_NO_ERROR;
}


/*****************************************************************************
* Name: MMCPrepareAndSetup
* R}hg[N쐬đM
*****************************************************************************/
MMC_CC MMCPrepareAndSetup(UINT32 Args, UINT16 Cmd, UINT16 noBlocks, UINT16 Resp)
{
    UINT16 crcCMD;

/*
*P3OUT |= 0x04;
*/
    crcCMD = calculateCmdCRC(Args, Cmd);

	/* Setup the command */
    mmcSendCommand(Args, Cmd, crcCMD);

    return MMC_NO_ERROR;
}


/*****************************************************************************
* Name: checkCardBusy
* MMCReadyԂɂȂ܂ŁADataOutMĎ
*****************************************************************************/
MMC_CC checkCardBusy(void)
{
	UINT16  ii;
	UCHAR   datState = 0;

	ii = 0xFFFF;

	while ( ii ) {
		/* BusyԂł́ADataOutLowɂȂĂ̂ŁA0x00MĂ */
		if ( !SPIExchangeData(DUMMY_DATA, &datState) )
			return YES;
		if ( (datState & 0xFF) )
			return NO; /* DataOutHiɂȂ̂ŁAReadyԂֈڍs */
		ii--;
		datState = 0;
	}

	/* J[hBusy甲Ȃ */
	return YES;
}

